//----------------------------------------------------------------------------
//  Project BmpMenuApp
//  
//  Copyright  1998. All Rights Reserved.
//
//  SUBSYSTEM:    BmpMenuApp Application
//  FILE:         bmpmenuapp.cpp
//  AUTHOR:       Dieter Windau
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TBmpMenuApp (TApplication).
//
//----------------------------------------------------------------------------
#include <owlext\pch.h>
#pragma hdrstop

#include <owl/buttonga.h>
#include <owl/statusba.h>
#include <owl/docmanag.h>
#include <owl/filedoc.h>
#include <owlext/bmpmenu.h>

#include "bmpmenuapp.h"
#include "bmpmenumdiclient.h"
#include "bmpmenumdichild.h"
#include "BmpMenuDecoratedMDIFrame.h"
#include "bmpmenueditview.h"
#include "bmpmenuappaboutdlg.h" // Definition of about dialog.

#include "bmpmenuapp.rh"        // Definition of all resources.

////////////////////////////////////////////////////////////////////////////////
#include <owlext/owlxlink.h>  // have to be only once
////////////////////////////////////////////////////////////////////////////////

const TestMenuItems = 17;

//{{TBmpMenuApp Implementation}}

//{{DOC_VIEW}}
DEFINE_DOC_TEMPLATE_CLASS(TFileDocument, TBmpMenuEditView, DocType1);
//{{DOC_VIEW_END}}

//{{DOC_MANAGER}}
DocType1 __dvt1(_T("All Files"), _T("*.*"), 0, _T("txt"), 
  dtAutoDelete | dtUpdateDir | dtOverwritePrompt);
//{{DOC_MANAGER_END}}

//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE2(TBmpMenuApp, TRecentFiles, TApplication)
//{{TBmpMenuAppRSP_TBL_BEGIN}}
  EV_OWLVIEW(dnCreate, EvNewView),
  EV_OWLVIEW(dnClose,  EvCloseView),
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
  EV_COMMAND(CM_HELPABOUTTEST, CmHelpAbout),
  EV_OWLDOCUMENT(dnCreate, EvOwlDocument),
  EV_OWLDOCUMENT(dnRename, EvOwlDocument),
  EV_REGISTERED(MruFileMessage, CmFileSelected),
//{{TBmpMenuAppRSP_TBL_END}}
  EV_COMMAND_ENABLE(CM_EXIT, CeExit),

  EV_COMMAND(CM_DRAWRECESSED, CmDrawRecessed),
  EV_COMMAND_ENABLE(CM_DRAWRECESSED, CeDrawRecessed),
  EV_COMMAND(CM_ACCINTOOLTIP, CmShowAccInTooltips),
  EV_COMMAND_ENABLE(CM_ACCINTOOLTIP, CeShowAccInTooltips),
  EV_COMMAND(CM_SHOWBITMAP, CmShowBackBitmap),
  EV_COMMAND_ENABLE(CM_SHOWBITMAP, CeShowBackBitmap),
  EV_COMMAND(CM_SPECIALRADIODOT, CmShowRadioBitmap),
  EV_COMMAND_ENABLE(CM_SPECIALRADIODOT, CeShowRadioBitmap),
  EV_COMMAND(CM_SPECIALCHECKBOX, CmShowCheckBitmap),
  EV_COMMAND_ENABLE(CM_SPECIALCHECKBOX, CeShowCheckBitmap),
  EV_COMMAND(CM_FIXEDFONT, CmShowFixedFont),
  EV_COMMAND_ENABLE(CM_FIXEDFONT, CeShowFixedFont),

  EV_COMMAND(CM_VIEW50, CmView50),
  EV_COMMAND_ENABLE(CM_VIEW50, CeView),
  EV_COMMAND(CM_VIEW100, CmView100),
  EV_COMMAND_ENABLE(CM_VIEW100, CeView),
  EV_COMMAND(CM_VIEW200, CmView200),
  EV_COMMAND_ENABLE(CM_VIEW200, CeView),

  EV_COMMAND(CM_TOOLBAR, CmToolbar),
  EV_COMMAND_ENABLE(CM_TOOLBAR, CeToolbar),
  EV_COMMAND(CM_STATUSBAR, CmStatusbar),
  EV_COMMAND_ENABLE(CM_STATUSBAR, CeStatusbar),
  EV_COMMAND_ENABLE(CM_DISABLEDRADIOITEM, CeDisabledRadioItem),
  EV_COMMAND(CM_BITMAPONOFF, CmBitmapOnOff),
  EV_COMMAND_ENABLE(CM_BITMAPONOFF, CeBitmapOnOff),
  EV_COMMAND(CM_DISABLEHELPMENU, CmDisableHelpMenu),
  EV_COMMAND_ENABLE(CM_DISABLEHELPMENU, CeDisableHelpMenu),

  EV_COMMAND(CM_CHANGETEXT, CmChangeText),
  EV_COMMAND_ENABLE(CM_CHANGETEXT, CeChangeText),

  EV_COMMAND(CM_INSERTITEM, CmInsertItem),
  EV_COMMAND_ENABLE(CM_INSERTITEM, CeInsertItem),

  EV_COMMAND(CM_REMOVEITEM, CmRemoveItem),
  EV_COMMAND_ENABLE(CM_REMOVEITEM, CeRemoveItem),

  EV_COMMAND(CM_TESTITEM2, CmTestItem2),
  EV_COMMAND(CM_TESTITEM4, CmTestItem4),
END_RESPONSE_TABLE;


//--------------------------------------------------------
// TBmpMenuApp
// ~~~~~
//
TBmpMenuApp::TBmpMenuApp():
  TApplication(_T("BmpMenuApp")), TRecentFiles(_T(".\\BmpMenuApp.ini"), 4)
{
  SetDocManager(new TDocManager(dmMDI, this));

  // INSERT>> Your constructor code here.
  iView = 1;
  bmpOnOff = bToolbar = bStatusbar = true;
  numCommands = 0;
  popupMenu = fileMenu = 0;
  bDisableTestMenu = false;
}

TBmpMenuApp::~TBmpMenuApp()
{
  // INSERT>> Your destructor code here.
  delete popupMenu;
  delete fileMenu;
}

void TBmpMenuApp::CreateGadgets(TDockableControlBar* cb, bool server)
{
  if (!server)
  {
    cb->Insert(*new TButtonGadget(CM_MDIFILENEW, CM_MDIFILENEW));
    cb->Insert(*new TButtonGadget(CM_MDIFILEOPEN, CM_MDIFILEOPEN));
    cb->Insert(*new TButtonGadget(CM_FILESAVE, CM_FILESAVE));
    cb->Insert(*new TSeparatorGadget(6));
  }

  cb->Insert(*new TButtonGadget(CM_EDITCUT, CM_EDITCUT));
  cb->Insert(*new TButtonGadget(CM_EDITCOPY, CM_EDITCOPY));
  cb->Insert(*new TButtonGadget(CM_EDITPASTE, CM_EDITPASTE));
  cb->Insert(*new TSeparatorGadget(6));
  cb->Insert(*new TButtonGadget(CM_EDITUNDO, CM_EDITUNDO));
  cb->Insert(*new TSeparatorGadget(6));
  cb->Insert(*new TButtonGadget(CM_EDITFIND, CM_EDITFIND));
  cb->Insert(*new TButtonGadget(CM_EDITFINDNEXT, CM_EDITFINDNEXT));

  // Add caption and fly-over help hints.
  //
  cb->SetCaption(_T("Toolbar"));
  cb->SetHintMode(TGadgetWindow::EnterHints);

  // Try OWL 6.0 flat style gadgets
  // TGadgetWindow::EnableFlatStyle(true);
}


void TBmpMenuApp::SetupSpeedBar(TDecoratedMDIFrame* frame)
{
  ApxHarbor = new THarbor(*frame);

  // Create default toolbar New and associate toolbar buttons with commands.
  //
  TDockableControlBar* cb = new TDockableControlBar(frame);
  CreateGadgets(cb);

  // Setup the toolbar ID used by OLE 2 for toolbar negotiation.
  //
  cb->Attr.Id = IDW_TOOLBAR;

  ApxHarbor->Insert(*cb, alTop);

  // Try to get MS-Office style margins
  //
  // cb->SetMargins(TMargins(TMargins::Pixels, 5,5,1,1));
  // cb->SetMargins(TMargins(TMargins::Pixels, 5,5,4,4));
}


//--------------------------------------------------------
// TBmpMenuApp
// ~~~~~
// Application main window construction & intialization.
//
void TBmpMenuApp::InitMainWindow()
{
  if (nCmdShow != SW_HIDE)
    nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;

  MdiClient = new TBmpMenuMDIClient(this);
  TBmpMenuDecoratedMDIFrame* frame = new TBmpMenuDecoratedMDIFrame(Name, 
    IDM_MDI, *MdiClient, true, this);

  nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;

  // Assign icons for this application.
  //
  frame->SetIcon(this, IDI_MDIAPPLICATION);
  frame->SetIconSm(this, IDI_MDIAPPLICATION);

  // Associate with the accelerator table.
  //
  frame->Attr.AccelTable = IDM_MDI;

  TStatusBar* sb = new TStatusBar(frame, TGadget::Recessed,
                                  TStatusBar::CapsLock        |
                                  TStatusBar::NumLock         |
                                  TStatusBar::ScrollLock);
  frame->Insert(*sb, TDecoratedFrame::Bottom);

  SetupSpeedBar(frame);

  SetMainWindow(frame);

  frame->SetMenuDescr(TMenuDescr(IDM_MDI));

  // load popup menu
  //
  popupMenu = new TMenu(LoadMenu(IDM_TESTPOPUP), AutoDelete);
}

void TBmpMenuApp::InitInstance()
{
  TApplication::InitInstance();

  // Hint: If you have popup menus that reside popup menus, they aren't remapped
  // correct. The reason for that is the following TBmpMenu function:
  // void RemapMenu(HMENU hMenu, bool isPopup=false)
  //
  fileMenu = new TMenu(LoadMenu(IDM_FILE), AutoDelete);
  if (fileMenu) {
    TBmpMenuDecoratedMDIFrame* frame =
      TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
    if (frame && frame->GetBmpMenu())
      frame->GetBmpMenu()->RemapMenu((HMENU)fileMenu->GetSubMenu(0), true);
    GetMainWindow()->AssignContextMenu(new TPopupMenu(fileMenu->GetSubMenu(0)));
  }
}

//--------------------------------------------------------
// TBmpMenuApp
// ~~~~~
// Response Table handlers:
//
void TBmpMenuApp::EvNewView(TView& view)
{
  TBmpMenuMDIClient* mdiClient = TYPESAFE_DOWNCAST(
    GetMainWindow()->GetClientWindow(), TBmpMenuMDIClient);
  if (mdiClient) {
    TBmpMenuMDIChild* child = new TBmpMenuMDIChild(*mdiClient, 0,
      view.GetWindow());

    // Set the menu descriptor for this view.
    //
    if (view.GetViewMenu())
      child->SetMenuDescr(*view.GetViewMenu());

    // Assign icons with this child window.
    //
    child->SetIcon(this, IDI_DOC);
    child->SetIconSm(this, IDI_DOC);

    child->Create();
  }
}

void TBmpMenuApp::EvCloseView(TView&)
{
}

//--------------------------------------------------------
// TBmpMenuApp
// ~~~~~~~~~~~
// Menu Help About BmpMenuApp command
void TBmpMenuApp::CmHelpAbout()
{
  // Show the modal dialog.
  //
  TBmpMenuAppAboutDlg(GetMainWindow()).Execute();
}

void TBmpMenuApp::EvOwlDocument(TDocument& doc)
{
  if (doc.GetDocPath())
    SaveMenuChoice(doc.GetDocPath());
}

int32 TBmpMenuApp::CmFileSelected(uint wp, int32)
{
  TAPointer<TCHAR> text = new TCHAR[_MAX_PATH];

  GetMenuText(wp, text, _MAX_PATH);
  TDocTemplate* tpl = GetDocManager()->MatchTemplate(text);
  if (tpl)
    GetDocManager()->CreateDoc(tpl, text);
  return 0;
}

void TBmpMenuApp::CeExit(TCommandEnabler& ce)
{
  // let TRecentFiles change the file menu
  //
  TRecentFiles::CeExit(ce);

  // after that, remap only the file menu to a bmp menu
  //
  TMenuItemEnabler* me = TYPESAFE_DOWNCAST(&ce, TMenuItemEnabler);
  if (me && me->GetMenu())
  {
    TBmpMenuDecoratedMDIFrame* frame =
      TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
    if (frame && frame->GetBmpMenu())
      frame->GetBmpMenu()->RemapMenu(me->GetMenu(), true);
  }
}

void TBmpMenuApp::CmDrawRecessed()
{
  TBmpMenuDecoratedMDIFrame* frame =
    TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
  if (frame && frame->GetBmpMenu())
  {
    frame->GetBmpMenu()->SetDrawRecessed(
      frame->GetBmpMenu()->IsDrawRecessed() ? false : true);
  }
}

void TBmpMenuApp::CeDrawRecessed(TCommandEnabler& ce)
{
  TBmpMenuDecoratedMDIFrame* frame =
    TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
  if (frame && frame->GetBmpMenu())
  {
    ce.SetCheck(frame->GetBmpMenu()->IsDrawRecessed());
  }
}

void TBmpMenuApp::CmShowAccInTooltips()
{
  TBmpMenuDecoratedMDIFrame* frame =
    TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
  if (frame && frame->GetBmpMenu())
  {
    frame->GetBmpMenu()->SetWithAccel(
      frame->GetBmpMenu()->IsWithAccel() ? false : true);
  }
}

void TBmpMenuApp::CeShowAccInTooltips(TCommandEnabler& ce)
{
  TBmpMenuDecoratedMDIFrame* frame =
    TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
  if (frame && frame->GetBmpMenu())
  {
    ce.SetCheck(frame->GetBmpMenu()->IsWithAccel());
  }
}

void TBmpMenuApp::CmShowBackBitmap()
{
  TBmpMenuDecoratedMDIFrame* frame =
    TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
  if (frame && frame->GetBmpMenu())
  {
    uint bmp = frame->GetBmpMenu()->IsBackBitmap() ? 0 : IDB_BACKGROUND;
    frame->GetBmpMenu()->SetBackBitmap(bmp);
    frame->DrawMenuBar();
  }
}

void TBmpMenuApp::CeShowBackBitmap(TCommandEnabler& ce)
{
  TBmpMenuDecoratedMDIFrame* frame =
    TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
  if (frame && frame->GetBmpMenu())
  {
    ce.SetCheck(frame->GetBmpMenu()->IsBackBitmap());
  }
}

void TBmpMenuApp::CmShowRadioBitmap()
{
  TBmpMenuDecoratedMDIFrame* frame =
    TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
  if (frame && frame->GetBmpMenu())
  {
    uint bmp = frame->GetBmpMenu()->IsRadioBitmap() ? 0 : IDB_DIAMONDRADIODOT;
    frame->GetBmpMenu()->SetRadioBitmap(bmp);
  }
}

void TBmpMenuApp::CeShowRadioBitmap(TCommandEnabler& ce)
{
  TBmpMenuDecoratedMDIFrame* frame =
    TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
  if (frame && frame->GetBmpMenu())
  {
    ce.SetCheck(frame->GetBmpMenu()->IsRadioBitmap());
  }
}

void TBmpMenuApp::CmShowCheckBitmap()
{
  TBmpMenuDecoratedMDIFrame* frame =
    TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
  if (frame && frame->GetBmpMenu())
  {
    uint bmp = frame->GetBmpMenu()->IsCheckBitmap() ? 0 : IDB_XCHECKBOX;
    frame->GetBmpMenu()->SetCheckBitmap(bmp);
  }
}

void TBmpMenuApp::CeShowCheckBitmap(TCommandEnabler& ce)
{
  TBmpMenuDecoratedMDIFrame* frame =
    TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
  if (frame && frame->GetBmpMenu())
  {
    ce.SetCheck(frame->GetBmpMenu()->IsCheckBitmap());
  }
}

void TBmpMenuApp::CmShowFixedFont()
{
  TBmpMenuDecoratedMDIFrame* frame =
    TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
  if (frame && frame->GetBmpMenu())
  {
    TFont* font = 0;
    if (frame->GetBmpMenu()->GetMenuFont() == 0)
      font = new TFont(_T("Courier New"), 20);
    frame->GetBmpMenu()->SetMenuFont(font);
    frame->DrawMenuBar();
  }
}

void TBmpMenuApp::CeShowFixedFont(TCommandEnabler& ce)
{
  TBmpMenuDecoratedMDIFrame* frame =
    TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
  if (frame && frame->GetBmpMenu())
  {
    ce.SetCheck(frame->GetBmpMenu()->GetMenuFont() == 0 ? false : true);
  }
}

void TBmpMenuApp::CmDisableHelpMenu()
{
  bDisableTestMenu = (bDisableTestMenu) ? false : true;
  if (GetMainWindow())
  {
    HMENU hMenu = GetMainWindow()->GetMenu();
    if (hMenu)
    {
      if (bDisableTestMenu)
        ::EnableMenuItem(hMenu, 5, MF_GRAYED | MF_BYPOSITION);
      else
        ::EnableMenuItem(hMenu, 5, MF_ENABLED | MF_BYPOSITION);
      GetMainWindow()->DrawMenuBar();  
    }
  }
}

void TBmpMenuApp::CeDisableHelpMenu(TCommandEnabler& ce)
{
  ce.SetCheck(bDisableTestMenu);
}

void TBmpMenuApp::CmView50()
{
  iView = 0;
}

void TBmpMenuApp::CmView100()
{
  iView = 1;
}

void TBmpMenuApp::CmView200()
{
  iView = 2;
}

void TBmpMenuApp::CeView(TCommandEnabler& ce)
{
  TMenuItemEnabler* me = TYPESAFE_DOWNCAST(&ce, TMenuItemEnabler);
  if (me && me->GetMenu())
  {
    ::CheckMenuRadioItem(me->GetMenu(), CM_VIEW50, CM_VIEW200,
      CM_VIEW50+iView, MF_BYCOMMAND);
  }
}

void TBmpMenuApp::CmToolbar()
{
  bToolbar = (bToolbar) ? false : true;
}

void TBmpMenuApp::CeToolbar(TCommandEnabler& ce)
{
//  ce.SetCheck(bToolbar);
  ce.SetCheck(false);
  ce.Enable(false);
}

void TBmpMenuApp::CmStatusbar()
{
  bStatusbar = (bStatusbar) ? false : true;
}

void TBmpMenuApp::CeStatusbar(TCommandEnabler& ce)
{
//  ce.SetCheck(bStatusbar);
  ce.SetCheck(true);
  ce.Enable(false);
}

void TBmpMenuApp::CeDisabledRadioItem(TCommandEnabler& ce)
{
  TMenuItemEnabler* me = TYPESAFE_DOWNCAST(&ce, TMenuItemEnabler);
  if (me && me->GetMenu())
  {
    ::CheckMenuRadioItem(me->GetMenu(), CM_DISABLEDRADIOITEM,
      CM_DISABLEDRADIOITEM,  CM_DISABLEDRADIOITEM, MF_BYCOMMAND);
  }
  ce.Enable(false);
}

void TBmpMenuApp::CmBitmapOnOff()
{
  bmpOnOff = (bmpOnOff) ? false : true;
}

void TBmpMenuApp::CeBitmapOnOff(TCommandEnabler& ce)
{
  ce.SetCheck(bmpOnOff);
}

void TBmpMenuApp::CmChangeText()
{
  numCommands++;
}

void TBmpMenuApp::CeChangeText(TCommandEnabler& ce)
{
  wsprintf(menutxt, _T("&Change Text: command %d"), numCommands);
  TMenuItemEnabler* me = TYPESAFE_DOWNCAST(&ce, TMenuItemEnabler);
  if (me && me->GetMenu())
  {
    TBmpMenuDecoratedMDIFrame* frame =
      TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
    if (frame && frame->GetBmpMenu())
      frame->GetBmpMenu()->SetMenuItemText(me->GetMenu(),
        me->GetPosition(), menutxt);
  }
}

void TBmpMenuApp::CmInsertItem()
{
  HMENU hMenu = GetMainWindow()->GetMenu();
  if (hMenu)
  {
    // Get the view menu
    //
    hMenu = GetSubMenu(hMenu, 3);
    if (hMenu)
    {
      int count = ::GetMenuItemCount(hMenu);
      if (count >= TestMenuItems)
      {
        if (count >= TestMenuItems+2)
        {
          // append popup menu and set one to the default item
          //
          TCHAR str[128];
          if (popupMenu && popupMenu->GetMenuString(0, str, 128, MF_BYPOSITION))
          {
            ::AppendMenu(hMenu, MF_POPUP|MF_STRING,
              (UINT)popupMenu->GetSubMenu(0), str);
            ::SetMenuDefaultItem(popupMenu->GetSubMenu(0), CM_TESTITEM4, false);
          }
        }
        else
        {
          // append separator (with id) and
          // simple string menu with associated bitmap
          //
          ::AppendMenu(hMenu, MF_SEPARATOR, CM_TESTITEM1, 0);
          ::AppendMenu(hMenu, MF_STRING, CM_TESTITEM2,
            _T("Test item &with bitmap...\tStrg+W"));
        }

        // remap with true, because the view menu is a submenu
        //
         TBmpMenuDecoratedMDIFrame* frame =
          TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
        if (frame && frame->GetBmpMenu())
          frame->GetBmpMenu()->RemapMenu(hMenu, true);
      }
    }
  }
}

void TBmpMenuApp::CeInsertItem(TCommandEnabler& ce)
{
  TMenuItemEnabler* me = TYPESAFE_DOWNCAST(&ce, TMenuItemEnabler);
  if (me && me->GetMenu())
  {
    int count = ::GetMenuItemCount(me->GetMenu());
    if (count != -1 && count < TestMenuItems+3)
    {
      ce.Enable(true);
      return;
    }
  }
  ce.Enable(false);
}

void TBmpMenuApp::CmRemoveItem()
{
  HMENU hMenu = GetMainWindow()->GetMenu();
  if (hMenu)
  {
    // Get the view menu
    //
    hMenu = GetSubMenu(hMenu, 3);
    if (hMenu)
    {
      int count = ::GetMenuItemCount(hMenu);
      if (count > TestMenuItems)
      {
        if (count > TestMenuItems+2)
        {
          // remove the popup menu (last item) but do not destroy them
          //
          int pos = ::GetMenuItemCount(hMenu);
          ::RemoveMenu(hMenu, pos-1, MF_BYPOSITION);
        }
        else
        {
          // delete menu item and separator by command
          //
          ::DeleteMenu(hMenu, CM_TESTITEM1, MF_BYCOMMAND);
          ::DeleteMenu(hMenu, CM_TESTITEM2, MF_BYCOMMAND);
        }

        // remap with true, because the view menu is a submenu
        //
        TBmpMenuDecoratedMDIFrame* frame =
          TYPESAFE_DOWNCAST(GetMainWindow(), TBmpMenuDecoratedMDIFrame);
        if (frame && frame->GetBmpMenu())
          frame->GetBmpMenu()->RemapMenu(hMenu, true);
      }
    }
  }
}

void TBmpMenuApp::CeRemoveItem(TCommandEnabler& ce)
{
  TMenuItemEnabler* me = TYPESAFE_DOWNCAST(&ce, TMenuItemEnabler);
  if (me && me->GetMenu())
  {
    int count = ::GetMenuItemCount(me->GetMenu());
    if (count != -1 && count > TestMenuItems-1)
    {
      ce.Enable(true);
      return;
    }
  }
  ce.Enable(false);
}

void TBmpMenuApp::CmTestItem2()
{
  GetMainWindow()->MessageBox(_T("Test item with bitmap command"));
}

void TBmpMenuApp::CmTestItem4()
{
  GetMainWindow()->MessageBox(_T("Default item command"));
}

int OwlMain(int, LPTSTR [])
{
  TBmpMenuApp   app;
  return app.Run();
}
