#include <owlext\pch.h>
#pragma hdrstop

#include <owlext/bmpmenu.h>
#include "BmpMenuDecoratedMDIFrame.h"

#include "bmpmenuapp.rh"            // Definition of all resources.

DEFINE_RESPONSE_TABLE1(TBmpMenuDecoratedMDIFrame, TDecoratedMDIFrame)
  EV_WM_INITMENU,
  EV_WM_INITMENUPOPUP,
  EV_WM_DRAWITEM,
  EV_WM_MEASUREITEM,
  EV_WM_MENUCHAR,
END_RESPONSE_TABLE;

TBmpMenuDecoratedMDIFrame::TBmpMenuDecoratedMDIFrame(
  LPCTSTR            title,
  TResId             menuResId,
  TBmpMenuMDIClient& clientWnd,
  bool               trackMenuSelection,
  TModule*           module):
  TDecoratedMDIFrame(title, menuResId, clientWnd, trackMenuSelection, module)
{
  BmpMenu = new TBmpMenu();

  // The following two commands has bitmap resources in the OWL 6.0 library
  // but they are should not shown in menus
  //
  //BmpMenu->SetMaskColor(TColor(RGB(0,128,0)));
  BmpMenu->ExcludeBitmap(CM_FILESAVEAS);
  BmpMenu->ExcludeBitmap(IDW_FIRSTMDICHILD);
  //BmpMenu->ExcludeBitmap(CM_HELPABOUT);
  //BmpMenu->IncludeBitmap(CM_HELPABOUT);

  // This is a test
  //
  BmpMenu->AddMapEntry(new TBmpMapEntry(CM_HELPABOUTTEST, CM_HELPABOUT));
  //BmpMenu->AddMapEntry(new TBmpMapEntry(CM_EDITUNDO, CM_HELPABOUT));
  //BmpMenu->SetMaskColor(TColor(0,255,0));
}

TBmpMenuDecoratedMDIFrame::~TBmpMenuDecoratedMDIFrame()
{
  delete BmpMenu;
}

int TBmpMenuDecoratedMDIFrame::GetHintText(uint id, LPTSTR buf, int size,
	THintText hintType) const
{
  // Call BmpMenu->GetHintText() to get tooltips
  //
  int retVal;
  if (BmpMenu)
  {
    retVal = BmpMenu->PreGetHintText(GetMenu(), id, buf, size, hintType);
    if (retVal != 0)
      return retVal;
  }
  return TDecoratedMDIFrame::GetHintText(id, buf, size, hintType);
}

void TBmpMenuDecoratedMDIFrame::SetupWindow()
{
	TDecoratedMDIFrame::SetupWindow();

  // Remap the copy of the MDI frame window system menu to a bitmap menu
  //
  // NOTE: If you remap the system menu in EvInitMenuPopup()
  //       it will be work too. I don't know why, but
  //       I think the MDI application EvInitMenuPopup() recieve
  //       automatic only a copy of the windows system menu
  //       and not the windows wide system menu
  // NOTE: At this point you remap only one time the system menu
  //       and so it's faster than in EvInitMenuPopup()
  //
  HMENU hMenu = GetSystemMenu(false);
  if (BmpMenu && hMenu)
    BmpMenu->RemapMenu(hMenu, true);
}

bool TBmpMenuDecoratedMDIFrame::SetMenu(HMENU hMenu)
{
  bool bValue = TDecoratedMDIFrame::SetMenu(hMenu);

  // Remap the main menu here one time, to get the right tooltips
  // from the beginning. If you don't do this, you have the right
  // tooltips at the first time then you click on a menu item ->EvInitMenu();
  //
  if (BmpMenu)
  	BmpMenu->RemapMenu(hMenu);

  return bValue;
}

void TBmpMenuDecoratedMDIFrame::EvInitMenu(HMENU hMenu)
{
  // Remap the MDI application main menu to a bitmap menu
  //
  if (BmpMenu)
  	BmpMenu->RemapMenu(hMenu);
  TDecoratedMDIFrame::EvInitMenu(hMenu);
}

void TBmpMenuDecoratedMDIFrame::EvInitMenuPopup(HMENU hPopupMenu,
  uint index, bool sysMenu)
{
//  if (BmpMenu)
//  	BmpMenu->RemapMenu(hPopupMenu, true);
  TDecoratedMDIFrame::EvInitMenuPopup(hPopupMenu, index, sysMenu);
}

void TBmpMenuDecoratedMDIFrame::EvDrawItem(uint ctrlId,
  DRAWITEMSTRUCT far& drawInfo)
{
  if (BmpMenu && BmpMenu->DrawItem(drawInfo))
    return;
  TDecoratedMDIFrame::EvDrawItem(ctrlId, drawInfo);
}

void TBmpMenuDecoratedMDIFrame::EvMeasureItem(uint ctrlId,
  MEASUREITEMSTRUCT far& measureInfo)
{
  if (BmpMenu && BmpMenu->MeasureItem(measureInfo))
    return;
  TDecoratedMDIFrame::EvMeasureItem(ctrlId, measureInfo);
}

int32 TBmpMenuDecoratedMDIFrame::EvMenuChar(uint nChar, uint menuType,
  HMENU hMenu)
{
  LRESULT lRes;
  if (BmpMenu && BmpMenu->FindKeyboardShortcut(nChar, menuType, hMenu, lRes))
    return lRes;
  return TDecoratedMDIFrame::EvMenuChar(nChar, menuType, hMenu);
}