//----------------------------------------------------------------------------
//  Project BmpMenuApp
//  
//  Copyright  1998. All Rights Reserved.
//
//  SUBSYSTEM:    BmpMenuApp Application
//  FILE:         bmpmenueditview.cpp
//  AUTHOR:       Dieter Windau
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TBmpMenuEditView (TEditView).
//
//----------------------------------------------------------------------------

#include <owlext\pch.h>
#pragma hdrstop

#include <stdio.h>
#include <owlext\bmpmenu.h>

#include "bmpmenueditview.h"
#include "bmpmenumdichild.h"

#include "bmpmenuapp.rh"

//{{TBmpMenuEditView Implementation}}

DEFINE_RESPONSE_TABLE1(TBmpMenuEditView, TEditView)
  EV_WM_CONTEXTMENU,
  EV_WM_DRAWITEM,
  EV_WM_MEASUREITEM,
  EV_WM_MENUCHAR,
END_RESPONSE_TABLE;

//--------------------------------------------------------
// TBmpMenuEditView
// ~~~~~~~~~~
// Construction/Destruction handling.
//
TBmpMenuEditView::TBmpMenuEditView(TDocument& doc, TWindow* parent)
:
  TEditView(doc, parent)
{
  // INSERT>> Your constructor code here.

  // Load a own text editor popup menu by resource,
  // because we can't remap the default windows text edidor popup menu
  //
  HMENU popmenu = 0;
  HMENU menu = GetApplication()->LoadMenu(IDM_EDITPOPUP);

  // Create popup menu
  //
  if (menu)
    popmenu = ::GetSubMenu(menu, 0);
  if (popmenu)
    ContextPopupMenu = new TPopupMenu(popmenu, AutoDelete);
}


TBmpMenuEditView::~TBmpMenuEditView()
{
  // INSERT>> Your destructor code here.

}

TBmpMenu* TBmpMenuEditView::GetBmpMenu()
{
  TBmpMenuMDIChild* child = TYPESAFE_DOWNCAST(Parent, TBmpMenuMDIChild);
  CHECK(child);
  return child->GetBmpMenu();
}

void TBmpMenuEditView::EvContextMenu(HWND childHwnd, int x, int y)
{
  // Remap the context menu of the edit window to a bitmap menu
  //
  if (GetBmpMenu() && GetContextMenu())
    GetBmpMenu()->RemapMenu(*GetContextMenu(), true);
  TEditView::EvContextMenu(childHwnd, x, y);
}

void TBmpMenuEditView::EvDrawItem(uint ctrlId, DRAWITEMSTRUCT far& drawInfo)
{
  if (GetBmpMenu() && GetBmpMenu()->DrawItem(drawInfo))
    return;
  TEditView::EvDrawItem(ctrlId, drawInfo);
}

void TBmpMenuEditView::EvMeasureItem(uint ctrlId, MEASUREITEMSTRUCT far& measureInfo)
{
  if (GetBmpMenu() && GetBmpMenu()->MeasureItem(measureInfo))
    return;
  TEditView::EvMeasureItem(ctrlId, measureInfo);
}



int32 TBmpMenuEditView::EvMenuChar(uint nChar, uint menuType, HMENU hMenu)
{
  LRESULT lRes;
  if (GetBmpMenu() && GetBmpMenu()->FindKeyboardShortcut(nChar, menuType, hMenu, lRes))
    return lRes;
  return TEditView::EvMenuChar(nChar, menuType, hMenu);
}


