//----------------------------------------------------------------------------
//  Project BmpMenuSDIApp
//  
//  Copyright  1998. All Rights Reserved.
//
//  SUBSYSTEM:    BmpMenuSDIApp Application
//  FILE:         bmpmenusdiapp.cpp
//  AUTHOR:       Dieter Windau
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TBmpMenuSDIApp (TApplication).
//
//----------------------------------------------------------------------------
#include <owl\pch.h>
#pragma hdrstop

#include <owl/buttonga.h>
#include <owl/docmanag.h>
#include <owl/statusba.h>
#include <owl/filedoc.h>

#include <classlib/filename.h>

#include "bmpmenusdiapp.h"
#include "bmpmenusdiappeditview.h"     // Definition of client class.
#include "bmpmenusdiappaboutdlg.h"     // Definition of about dialog.

#include <owlext\bmpmenu.h>

#include "bmpmenusdiapp.rh"            // Definition of all resources.

////////////////////////////////////////////////////////////////////////////////
#include <owlext/owlxlink.h>  // have to be only once
////////////////////////////////////////////////////////////////////////////////

//{{TBmpMenuSDIApp Implementation}}

//{{DOC_VIEW}}
DEFINE_DOC_TEMPLATE_CLASS(TFileDocument, TBmpMenuSDIAppEditView, DocType1);
//{{DOC_VIEW_END}}

//{{DOC_MANAGER}}
DocType1 __dvt1(_T("All Files"), _T("*.*"), 0, _T("txt"), 
  dtAutoDelete | dtUpdateDir | dtOverwritePrompt);
//{{DOC_MANAGER_END}}

//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE2(TBmpMenuSDIApp, TRecentFiles, TApplication)
//{{TBmpMenuSDIAppRSP_TBL_BEGIN}}
  EV_OWLVIEW(dnCreate, EvNewView),
  EV_OWLVIEW(dnClose,  EvCloseView),
  EV_COMMAND(CM_FILESEND, CmFileSend),
  EV_COMMAND_ENABLE(CM_FILESEND, CeFileSend),
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
  EV_COMMAND(CM_FILEPRINT, CmFilePrint),
  EV_COMMAND(CM_FILEPRINTERSETUP, CmFilePrintSetup),
  EV_COMMAND(CM_FILEPRINTPREVIEW, CmFilePrintPreview),
  EV_COMMAND_ENABLE(CM_FILEPRINT, CmPrintEnable),
  EV_COMMAND_ENABLE(CM_FILEPRINTERSETUP, CmPrintEnable),
  EV_COMMAND_ENABLE(CM_FILEPRINTPREVIEW, CmPrintEnable),
  EV_WM_WININICHANGE,
  EV_OWLDOCUMENT(dnCreate, EvOwlDocument),
  EV_OWLDOCUMENT(dnRename, EvOwlDocument),
  EV_REGISTERED(MruFileMessage, CmFileSelected),
//{{TBmpMenuSDIAppRSP_TBL_END}}
END_RESPONSE_TABLE;


//--------------------------------------------------------
// TBmpMenuSDIApp
// ~~~~~
//
TBmpMenuSDIApp::TBmpMenuSDIApp(): 
  TApplication(_T("BmpMenuSDIApp")), 
  TRecentFiles(_T(".\\BmpMenuSDIApp.ini"), 4)
{
  Printer = 0;
  Printing = 0;

  SetDocManager(new TDocManager(dmSDI, this));

  ApxMail = new TMailer();

  // INSERT>> Your constructor code here.
}

TBmpMenuSDIApp::~TBmpMenuSDIApp()
{
  delete Printer;

  delete ApxMail;

  // INSERT>> Your destructor code here.
}


void TBmpMenuSDIApp::CreateGadgets(TDockableControlBar* cb, bool server)
{
  if (!server)
  {
    cb->Insert(*new TButtonGadget(CM_FILENEW, CM_FILENEW));
    cb->Insert(*new TButtonGadget(CM_FILEOPEN, CM_FILEOPEN));
    cb->Insert(*new TButtonGadget(CM_FILESAVE, CM_FILESAVE));
    cb->Insert(*new TSeparatorGadget(6));
  }

  cb->Insert(*new TButtonGadget(CM_EDITCUT, CM_EDITCUT));
  cb->Insert(*new TButtonGadget(CM_EDITCOPY, CM_EDITCOPY));
  cb->Insert(*new TButtonGadget(CM_EDITPASTE, CM_EDITPASTE));
  cb->Insert(*new TSeparatorGadget(6));
  cb->Insert(*new TButtonGadget(CM_EDITUNDO, CM_EDITUNDO));
  cb->Insert(*new TSeparatorGadget(6));
  cb->Insert(*new TButtonGadget(CM_EDITFIND, CM_EDITFIND));
  cb->Insert(*new TButtonGadget(CM_EDITFINDNEXT, CM_EDITFINDNEXT));

  if (!server)
  {
    cb->Insert(*new TSeparatorGadget(6));
    cb->Insert(*new TButtonGadget(CM_FILEPRINT, CM_FILEPRINT));
    cb->Insert(*new TButtonGadget(CM_FILEPRINTPREVIEW, CM_FILEPRINTPREVIEW));
  }

  // Add caption and fly-over help hints.
  //
  cb->SetCaption(_T("Toolbar"));
  cb->SetHintMode(TGadgetWindow::EnterHints);
}

void TBmpMenuSDIApp::SetupSpeedBar(TDecoratedFrame* frame)
{
  ApxHarbor = new THarbor(*frame);

  // Create default toolbar New and associate toolbar buttons with commands.
  //
  TDockableControlBar* cb = new TDockableControlBar(frame);
  CreateGadgets(cb);

  // Setup the toolbar ID used by OLE 2 for toolbar negotiation.
  //
  cb->Attr.Id = IDW_TOOLBAR;

  ApxHarbor->Insert(*cb, alTop);
}

//--------------------------------------------------------
// TBmpMenuSDIApp
// ~~~~~
// Application main window construction & intialization.
//
void TBmpMenuSDIApp::InitMainWindow()
{
  if (nCmdShow != SW_HIDE)
    nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;

  TSDIDecFrame* frame = new TSDIDecFrame(0, GetName(), 0, true, this);

  // Assign icons for this application.
  //
  frame->SetIcon(this, IDI_SDIAPPLICATION);
  frame->SetIconSm(this, IDI_SDIAPPLICATION);

  // Menu associated with window and accelerator table associated with table.
  //
  frame->AssignMenu(IDM_SDI);

  // Associate with the accelerator table.
  //
  frame->Attr.AccelTable = IDM_SDI;

  TStatusBar* sb = new TStatusBar(frame, TGadget::Recessed,
                                  TStatusBar::CapsLock        |
                                  TStatusBar::NumLock         |
                                  TStatusBar::ScrollLock);
  frame->Insert(*sb, TDecoratedFrame::Bottom);

  SetupSpeedBar(frame);

  SetMainWindow(frame);

  frame->SetMenuDescr(TMenuDescr(IDM_SDI));

}

//--------------------------------------------------------
// TBmpMenuSDIApp
// ~~~~~
// Application instance initialization.
//
void TBmpMenuSDIApp::InitInstance()
{
  TApplication::InitInstance();
  GetDocManager()->CreateAnyDoc(0, dtNewDoc);
}

//--------------------------------------------------------
// TBmpMenuSDIApp
// ~~~~~
// Response Table handlers:
//
void TBmpMenuSDIApp::EvNewView(TView& view)
{
  GetMainWindow()->SetClientWindow(view.GetWindow());
  if (!view.IsOK())
    GetMainWindow()->SetClientWindow(0);
  else if (view.GetViewMenu())
    GetMainWindow()->MergeMenu(*view.GetViewMenu());
}


void TBmpMenuSDIApp::EvCloseView(TView&)
{
  GetMainWindow()->SetClientWindow(0);
  GetMainWindow()->SetCaption(_T("BmpMenuSDIApp"));
}


//{{TSDIDecFrame Implementation}}

DEFINE_RESPONSE_TABLE1(TSDIDecFrame, TDecoratedFrame)
  EV_WM_INITMENU,
  EV_WM_DRAWITEM,
  EV_WM_MEASUREITEM,
  EV_WM_MENUCHAR,
END_RESPONSE_TABLE;

TSDIDecFrame::TSDIDecFrame(TWindow* parent, LPCTSTR title, TWindow* clientWnd,
  bool trackMenuSelection, TModule* module)
:
  TDecoratedFrame(parent, title, clientWnd, trackMenuSelection, module)
{
  // INSERT>> Your constructor code here.
  //BmpMenu = new TBmpMenu(new TModule("msstyres.dll", true));
  BmpMenu = new TBmpMenu();
  BmpMenu->ExcludeBitmap(CM_FILESAVEAS);
}

TSDIDecFrame::~TSDIDecFrame()
{
  // INSERT>> Your destructor code here.
  delete BmpMenu;
}

int TSDIDecFrame::GetHintText(uint id, LPTSTR buf, int size,
	THintText hintType) const
{
  // Call BmpMenu->GetHintText() to get tooltips
  //
  int retVal;
  if (BmpMenu)
  {
    retVal = BmpMenu->PreGetHintText(GetMenu(), id, buf, size, hintType);
    if (retVal != 0)
      return retVal;
  }
  return TDecoratedFrame::GetHintText(id, buf, size, hintType);
}

void TSDIDecFrame::SetupWindow()
{
	TDecoratedFrame::SetupWindow();

  // Here you can remap the copy of the system menu
  // NOTE: If you remap the system menu with EvInitMenuPopup
  //       You remap the windows wide system menu and
  //       the system menu of (e.g. Explorer) is destroyed
  //
  HMENU hMenu = GetSystemMenu(false);
  if (BmpMenu && hMenu)
    BmpMenu->RemapMenu(hMenu, true);

  // Remap the main menu here one time, to get the right tooltips
  // from the beginning. If you don't do this, you have the right
  // tooltips at the first time then you click on a menu item ->EvInitMenu();
  //
  hMenu = GetMenu();
  if (BmpMenu && hMenu)
    BmpMenu->RemapMenu(hMenu);
}

void TSDIDecFrame::EvInitMenu(HMENU hMenu)
{
  // remap the SDI application main menu
  //
  if (BmpMenu)
  	BmpMenu->RemapMenu(hMenu);
  TDecoratedFrame::EvInitMenu(hMenu);
}

void TSDIDecFrame::EvDrawItem(uint ctrlId, DRAWITEMSTRUCT far& drawInfo)
{
  if (BmpMenu && BmpMenu->DrawItem(drawInfo))
    return;
  TDecoratedFrame::EvDrawItem(ctrlId, drawInfo);
}

void TSDIDecFrame::EvMeasureItem(uint ctrlId, MEASUREITEMSTRUCT far& measureInfo)
{
  if (BmpMenu && BmpMenu->MeasureItem(measureInfo))
    return;
  TDecoratedFrame::EvMeasureItem(ctrlId, measureInfo);
}

int32 TSDIDecFrame::EvMenuChar(uint nChar, uint menuType, HMENU hMenu)
{
  LRESULT lRes;
  if (BmpMenu && BmpMenu->FindKeyboardShortcut(nChar, menuType, hMenu, lRes))
    return lRes;
  return TDecoratedFrame::EvMenuChar(nChar, menuType, hMenu);
}

//--------------------------------------------------------
// TBmpMenuSDIApp
// ~~~~~~~~~~
// Menu File Print command
//
void TBmpMenuSDIApp::CmFilePrint()
{
  // Create Printer object if not already created.
  //
  if (!Printer)
    Printer = new TPrinter(this);

  TDocument* currentDoc = GetDocManager()->GetCurrentDoc();

  TAPointer<TCHAR> docName = new TCHAR[_MAX_PATH];

  if (currentDoc->GetTitle())
    _tcscpy(docName, currentDoc->GetTitle());
  else
    _tcscpy(docName, _T("Document"));

  // Create Printout window and set characteristics.
  //
  TApxPrintout printout(Printer, docName, GetMainWindow()->GetClientWindow());
  printout.SetBanding(true);

  Printing++;

  // Bring up the Print dialog and print the document.
  //
  Printer->Print(GetWindowPtr(GetActiveWindow()), printout, true);

  Printing--;
}

//--------------------------------------------------------
// TBmpMenuSDIApp
// ~~~~~~~~~~
// Menu File Print Setup command
//
void TBmpMenuSDIApp::CmFilePrintSetup()
{
  if (!Printer)
    Printer = new TPrinter(this);

  // Bring up the Print Setup dialog.
  //
  Printer->Setup(GetMainWindow());
}

//--------------------------------------------------------
// TBmpMenuSDIApp
// ~~~~~~~~~~
// Menu File Print Preview command
//
void TBmpMenuSDIApp::CmFilePrintPreview()
{
  TSDIDecFrame* sdiFrame = TYPESAFE_DOWNCAST(GetMainWindow(), TSDIDecFrame);
  if (sdiFrame)
  {
    if (!Printer)
      Printer = new TPrinter(this);

    Printing++;

    TApxPreviewWin* prevW = new TApxPreviewWin(
      sdiFrame, Printer, sdiFrame->GetClientWindow(), _T("Print Preview"),
      new TLayoutWindow((TWindow*)0));
    prevW->Create();

    // Here we resize the preview window to take the size of the MainWindow,
    // then hide the MainWindow.
    //
    TFrameWindow * mainWindow =  GetMainWindow();
    TRect r = mainWindow->GetWindowRect();
    prevW->MoveWindow(r);
    prevW->ShowWindow(SW_SHOWNORMAL);
    mainWindow->ShowWindow(SW_HIDE);

    BeginModal(GetMainWindow());

    // After the user closes the preview Window, we take its size and
    // use it to size the MainWindow, then show the MainWindow again.
    //
    r = prevW->GetWindowRect();
    mainWindow->MoveWindow(r);
    mainWindow->ShowWindow(SW_SHOWNORMAL);

    Printing--;

    // Now that printing is off we can invalidate
    // because the edit window to repaint.
    //
    GetMainWindow()->SetRedraw(true);
    GetMainWindow()->Invalidate();

    // We must destroy the preview window explicitly.  Otherwise, the window
    // will not be destroyed until it's parent the MainWindow is destroyed.
    //
    prevW->Destroy();
    delete prevW;
  }
}

//--------------------------------------------------------
// TBmpMenuSDIApp
// ~~~~~~~~~~
// Menu enabler used by Print, Print Setup and Print Preview.
//
void TBmpMenuSDIApp::CmPrintEnable(TCommandEnabler& tce)
{
  // If we have a Printer already created just test if all is okay.
  // Otherwise create a Printer object and make sure the printer really
  // exists and then delete the Printer object.
  //
  if (!Printer)
  {
    Printer = new TPrinter(this);
      tce.Enable(!Printer->GetSetup().Error);
  }
  else
    tce.Enable(!Printer->GetSetup().Error);
}


void TBmpMenuSDIApp::CeFileSend(TCommandEnabler& ce)
{
  ce.Enable((GetDocManager()->GetCurrentDoc() != 0)
            && ApxMail->IsMAPIAvailable());
}

void TBmpMenuSDIApp::CmFileSend ()
{
  // Check to see if a document exists
  //
  TDocument* currentDoc = GetDocManager()->GetCurrentDoc();

  if (currentDoc)
  {
    TAPointer<TCHAR> savedPath = new TCHAR[_MAX_PATH];
    TAPointer<TCHAR> docName = new TCHAR[_MAX_PATH];

    bool dirtyState = currentDoc->IsDirty();

    if (currentDoc->GetDocPath())
      _tcscpy(savedPath, currentDoc->GetDocPath());
    else
      _tcscpy(savedPath, _T(""));

    if (currentDoc->GetTitle())
      _tcscpy(docName, currentDoc->GetTitle());
    else
      _tcscpy(docName, _T("Document"));

    TFileName tempFile(TFileName::TempFile);

    currentDoc->SetDocPath(tempFile.Canonical().c_str());
    currentDoc->Commit(true);

    currentDoc->SetDocPath(savedPath);
    currentDoc->SetDirty(dirtyState);

    ApxMail->SendDocuments(GetMainWindow(), tempFile.Canonical().c_str(),
      docName, false);

    tempFile.Remove();
  }
}

//--------------------------------------------------------
// TBmpMenuSDIApp
// ~~~~~~~~~~~
//
// Menu Help About BmpMenuSDIApp command
void TBmpMenuSDIApp::CmHelpAbout()
{
  // Show the modal dialog.
  //
  TBmpMenuSDIAppAboutDlg(GetMainWindow()).Execute();
}

void TBmpMenuSDIApp::EvOwlDocument(TDocument& doc)
{
  if (doc.GetDocPath())
    SaveMenuChoice(doc.GetDocPath());
}

int32 TBmpMenuSDIApp::CmFileSelected(uint wp, int32)
{
  TAPointer<TCHAR> text = new TCHAR[_MAX_PATH];

  GetMenuText(wp, text, _MAX_PATH);
  TDocTemplate* tpl = GetDocManager()->MatchTemplate(text);
  if (tpl)
    GetDocManager()->CreateDoc(tpl, text);
  return 0;
}

void TBmpMenuSDIApp::EvWinIniChange(LPTSTR section)
{
  if (_tcscmp(section, _T("windows")) == 0)
  {
    // If the device changed in the WIN.INI file then the printer
    // might have changed.  If we have a TPrinter(Printer) then
    // check and make sure it's identical to the current device
    // entry in WIN.INI.
    //
    if (Printer)
    {
      const int bufferSize = 255;
      TCHAR printDBuffer[bufferSize];
      LPTSTR printDevice = printDBuffer;
      LPTSTR devName;
      LPTSTR driverName = 0;
      LPTSTR outputName = 0;
      if (::GetProfileString(_T("windows"), _T("device"), _T(""),
          printDevice, bufferSize))
      {
        // The string which should come back is something like:
        //
        //      HP LaserJet III,hppcl5a,LPT1:
        //
        // Where the format is:
        //
        //      devName,driverName,outputName
        //
        devName = printDevice;
        while (*printDevice)
        {
          if (*printDevice == _T(','))
          {
            *printDevice++ = 0;
            if (!driverName)
              driverName = printDevice;
            else
              outputName = printDevice;
          }
          else
#if defined(UNICODE)
            printDevice++;
#else
            printDevice = ::AnsiNext(printDevice);
#endif
        }

        if (Printer->GetSetup().Error != 0                ||
            _tcscmp(devName, Printer->GetSetup().GetDeviceName()) != 0  ||
            _tcscmp(driverName, Printer->GetSetup().GetDriverName()) != 0 ||
            _tcscmp(outputName, Printer->GetSetup().GetOutputName()) != 0 )
        {
          // New printer installed so get the new printer device now.
          //
          delete Printer;
          Printer = new TPrinter(this);
        }
      }
      else
      {
        // No printer installed(GetProfileString failed).
        //
        delete Printer;
        Printer = new TPrinter(this);
      }
    }
  }
}

int OwlMain(int, LPTSTR [])
{
  TBmpMenuSDIApp   app;
  return app.Run();
}