//----------------------------------------------------------------------------
//  Project BmpMenuSDIApp
//  
//  Copyright  1998. All Rights Reserved.
//
//  SUBSYSTEM:    BmpMenuSDIApp Application
//  FILE:         bmpmenusdiapp.h
//  AUTHOR:       Dieter Windau
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for TBmpMenuSDIApp (TApplication).
//
//----------------------------------------------------------------------------
#if !defined(bmpmenusdiapp_h)
#define bmpmenusdiapp_h

#if !defined(OWL_CONTROLB_H)
#include <owl/controlb.h>
#endif
#if !defined(OWL_DOCKING_H)
#include <owl/docking.h>
#endif
#if !defined(OWL_MAILER_H)
#include <owl/mailer.h>
#endif
#if !defined(OWL_PRINTER_H)
#include <owl/printer.h>
#endif
#if !defined(OWL_RCNTFILE_H)
#include <owl/rcntfile.h>
#endif

#if !defined(apxprint_h)
#include "apxprint.h"
#endif
#if !defined(apxprev_h)
#include "apxprev.h"
#endif

class TBmpMenu;

//
// FrameWindow must be derived to override Paint for Preview and Print.
//
//{{TDecoratedFrame = TSDIDecFrame}}
class TSDIDecFrame : public TDecoratedFrame
{
  public:
    TSDIDecFrame(TWindow* parent, LPCTSTR title, TWindow* clientWnd, 
      bool trackMenuSelection = false, TModule* module = 0);
    ~TSDIDecFrame();

    TBmpMenu* GetBmpMenu() { return BmpMenu; }

    virtual int GetHintText(uint id, LPTSTR buf, int size, 
      THintText hintType) const;
    
  protected:
    virtual void SetupWindow();
    
    void  EvInitMenu(HMENU hMenu);
    void  EvDrawItem(uint ctrlId, DRAWITEMSTRUCT far& drawInfo);
    void  EvMeasureItem(uint ctrlId, MEASUREITEMSTRUCT far& measureInfo);

    int32 EvMenuChar(uint nChar, uint menuType, HMENU hMenu);

  protected:
	  TBmpMenu* BmpMenu;

  DECLARE_RESPONSE_TABLE(TSDIDecFrame);    
};    //{{TSDIDecFrame}}


//{{TApplication = TBmpMenuSDIApp}}
class TBmpMenuSDIApp : public TApplication, public TRecentFiles
{
  private:
    void SetupSpeedBar(TDecoratedFrame* frame);

  public:
    TBmpMenuSDIApp();
    virtual ~TBmpMenuSDIApp();

    void CreateGadgets(TDockableControlBar* cb, bool server = false);
    THarbor*        ApxHarbor;

    // Public data members used by the print menu
    // commands and Paint routine in MDIChild.
    //
    TPrinter*       Printer;  // Printer support.
    int             Printing; // Printing in progress.

    TMailer*        ApxMail;

//{{TBmpMenuSDIAppVIRTUAL_BEGIN}}
  public:
    virtual void InitMainWindow();
    virtual void InitInstance();
//{{TBmpMenuSDIAppVIRTUAL_END}}

//{{TBmpMenuSDIAppRSP_TBL_BEGIN}}
  protected:
    void EvNewView(TView& view);
    void EvCloseView(TView& view);
    void CeFileSend(TCommandEnabler& ce);
    void CmFileSend();
    void CmHelpAbout();
    void CmFilePrint();
    void CmFilePrintSetup();
    void CmFilePrintPreview();
    void CmPrintEnable(TCommandEnabler& tce);
    void EvWinIniChange(LPTSTR section);
    void EvOwlDocument(TDocument& doc);
    int32 CmFileSelected(uint wp, int32 lp);
//{{TBmpMenuSDIAppRSP_TBL_END}}
DECLARE_RESPONSE_TABLE(TBmpMenuSDIApp);
};    //{{TBmpMenuSDIApp}}

#endif  // bmpmenusdiapp_h sentry.
