//----------------------------------------------------------------------------
//  Project BmpMenuSDIApp
//  
//  Copyright  1998. All Rights Reserved.
//
//  SUBSYSTEM:    BmpMenuSDIApp Application
//  FILE:         bmpmenusdiappeditview.cpp
//  AUTHOR:       Dieter Windau
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TBmpMenuSDIAppEditView (TEditView).
//
//----------------------------------------------------------------------------

#include <owl\pch.h>
#pragma hdrstop

#include <stdio.h>

#include "bmpmenusdiapp.h"
#include "bmpmenusdiappeditview.h"

//{{TBmpMenuSDIAppEditView Implementation}}

//
// Build a response table for all messages/commands handled
// by TBmpMenuSDIAppEditView derived from TEditView.
//
DEFINE_RESPONSE_TABLE1(TBmpMenuSDIAppEditView, TEditView)
//{{TBmpMenuSDIAppEditViewRSP_TBL_BEGIN}}
  EV_WM_GETMINMAXINFO,
//{{TBmpMenuSDIAppEditViewRSP_TBL_END}}
END_RESPONSE_TABLE;


//--------------------------------------------------------
// TBmpMenuSDIAppEditView
// ~~~~~~~~~~
// Construction/Destruction handling.
//
TBmpMenuSDIAppEditView::TBmpMenuSDIAppEditView(TDocument& doc, TWindow* parent)
:
  TEditView(doc, parent)
{
  // INSERT>> Your constructor code here.

}


TBmpMenuSDIAppEditView::~TBmpMenuSDIAppEditView()
{
  // INSERT>> Your destructor code here.

}


//
// Paint routine for Window, Printer, and PrintPreview for a TEditView client.
//
void TBmpMenuSDIAppEditView::Paint(TDC& dc, bool, TRect& rect)
{
  TBmpMenuSDIApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TBmpMenuSDIApp);
  if (theApp) {
    // Only paint if we're printing and we have something to paint, otherwise do nothing.
    //
    if (theApp->Printing && theApp->Printer && !rect.IsEmpty()) {
      // Use pageSize to get the size of the window to render into.  For a Window it's the client area,
      // for a printer it's the printer DC dimensions and for print preview it's the layout window.
      TSize   pageSize(rect.right - rect.left, rect.bottom - rect.top);

      HFONT   hFont = (HFONT)GetWindowFont();
      TFont   font(_T("Arial"), -12);
      if (!hFont)
        dc.SelectObject(font);
      else
        dc.SelectObject(TFont(hFont));

      TEXTMETRIC  tm;
      int fHeight = dc.GetTextMetrics(tm) ? tm.tmHeight + tm.tmExternalLeading : 10;

      // How many lines of this font can we fit on a page.
      //
      int linesPerPage = MulDiv(pageSize.cy, 1, fHeight);
      if (linesPerPage) {
        TPrintDialog::TData& printerData = theApp->Printer->GetSetup();

        int maxPg = ((GetNumLines() / linesPerPage) + 1.0);

        // Compute the number of pages to print.
        //
        printerData.MinPage = 1;
        printerData.MaxPage = maxPg;

        // Do the text stuff:
        //
        int   fromPage = printerData.FromPage == -1 ? 1 : printerData.FromPage;
        int   toPage = printerData.ToPage == -1 ? 1 : printerData.ToPage;
        int   currentPage = fromPage;
        TAPointer<TCHAR> buffer = new TCHAR[255];

        while (currentPage <= toPage) {
          int startLine = (currentPage - 1) * linesPerPage;
          int lineIdx = 0;
          while (lineIdx < linesPerPage) {
            // If the string is no longer valid then there's nothing more to display.
            //
            if (!GetLine(buffer, 255, startLine + lineIdx))
              break;
            dc.TabbedTextOut(TPoint(0, lineIdx * fHeight), buffer, _tcslen(buffer), 0, 0, 0);
            lineIdx++;
          }
          currentPage++;
        }
      }
    }
  }
}


void TBmpMenuSDIAppEditView::EvGetMinMaxInfo(MINMAXINFO far& minmaxinfo)
{
  TBmpMenuSDIApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TBmpMenuSDIApp);
  if (theApp) {
    if (theApp->Printing) {
      minmaxinfo.ptMaxSize = TPoint(32000, 32000);
      minmaxinfo.ptMaxTrackSize = TPoint(32000, 32000);
      return;
    }
  }
  TEditView::EvGetMinMaxInfo(minmaxinfo);
}
